/*
 * Pulser_Control.cpp
 *
 * Created: 03.02.2017 11:16:09
 * Actual: 11.01.2018
 *  Author: Jochen Feldhaar, AK Heilemann, IPTC, Goethe-Uni Frankfurt
 */ 

/*
Connections of ports

Port A: Bit 0	Power_Fault			Input
		Bit 1	Ena_HV_Relay		Output
		Bit 2	Behlke_Fault		Input
		Bit 3	Led 48				Output
		Bit 4	Led 49				Output
		Bit 5	Led 50				Output
		Bit 6	Led 51				Output
		Bit 7	Led 52				Output
		
Port B: Bit 0	Trig_Ena			Output
		Bit 1	SAW_End//SCK		Input
		Bit 2	Sample_End//MOSI	Output
		Bit 3	//MISO				Input/Output
		Bit 4	HV_Cur_ADC_/Conv	Output	
		Bit 5	HV_Cur_ADC_SDO		Input
		Bit 6	Puls_ADC_/Conv		Output
		Bit 7	Puls_ADC_SDO		Input
		
Port C: Bit 0	DB0					Output
		Bit 1	DB1					Output
		Bit 2	DB2					Output
		Bit 3	DB3					Output
		Bit 4	DB4					Output
		Bit 5	DB5					Output
		Bit 6	DB6					Output
		Bit 7	DB7					Output
		
Port D: Bit 0	595CLK				Output
		Bit 1	595DATA				Output
		Bit 2	595STR				Output
		Bit 3	165STR				Output
		Bit 4	165SCK				Output
		Bit 5	165DAT				Input
		Bit 6	E					Output
		Bit 7	RS					Output
		
Port E: Bit 0	RXD					Seriell
		Bit 1	TXD					Seriell
		Bit 2	DAC_DIN				Output
		Bit 3	DAC_SCLK			Output
		Bit 4	Time_DAC_/Sync		Output
		Bit 5	HV_DAC_/Sync		Output
		Bit 6	Puls_Cur_ADC_/Conv	Output
		Bit 7	Puls_Cur_ADC_SDO	Input
		
Port F: Bit 0	ADC_CLK				Output
		Bit 1	Samp_ADC_SDO		Input
		Bit 2	Samp_ADC_/Conv		Output
		Bit 3	HV_ADC_/Conv		Output
		Bit 4	HV_ADC_SDO			Input
		Bit 5	Count_Clock			Output
		Bit 6	Count_Serdata		Output
		Bit 7	Count_Strobe		Output
		
Port G: Bit 0	HV_On_/Off			Output
		Bit 1	HV_Pol_SW			Output
		Bit 2	Puls_Inh			Output
		Bit 3	Fault_Off			Output
		Bit 4	/Buzzer				Output
		



*/
#define F_CPU 16000000UL


// Includes go here
#include <avr/io.h>
#include <avr/interrupt.h>
#include <string.h>
#include <util/delay.h>



// Function Prototypes go here
void startup(void);
void beep(void);
int wr_voltdac(uint8_t);
void wr_sampdac(uint8_t);
void disp_erase(void);
void disp_init(void);
void disp_write(char wtext[26], int row, int column);
void shortwait(unsigned int delay);
ISR (TIMER2_OVF_vect);
ISR (TIMER1_OVF_vect);
ISR (TIMER0_OVF_vect);
void sr_write(void);
void wait_msec(uint16_t);
void wait_sec(uint16_t);
void win_develop(void);
void vr_volt_bb(void);




// Variable Declarations (global) go here
uint8_t Dac_Puls_Curr_Mode; // D16 and D17 for the DACs
uint8_t Dac_Sampling_Mode;
uint8_t Dac_HV_Mode;
uint8_t Dac_Puls_Volt_Mode;
uint16_t Dac_Puls_Curr_Value;
uint16_t Dac_Sampling_Value;
uint16_t Dac_HV_Value;
uint16_t Dac_Puls_Volt_Value;
uint8_t tempdac[26];
char wtext[26];
volatile unsigned int sh_wait;
unsigned int timecount = 5;
unsigned int delaycount = 5;
unsigned int sec = 0;
unsigned int min = 0;
unsigned int hour = 0;
unsigned int sec_old = 0;
unsigned int min_old = 0;
unsigned int hour_old = 0;
unsigned int count_val;
uint8_t Set_Trig_Imp;
uint8_t HV_Sup_Rel;
uint8_t HV_Meas_Pol;
uint8_t C_Big;
uint8_t C_Mid;
uint8_t Behl__Inh;
uint8_t Sreg_Sort[17];
uint8_t Led_Pos;
uint8_t Led_Neg;
uint8_t Led_Remote;
uint8_t Set_RW;
uint8_t Led_Trig;
uint8_t Led_Puls;
uint8_t Led_HV;
uint8_t Enc_A;
uint8_t Enc_B;
uint8_t Enc_But;
uint8_t But_Local;
uint8_t Key_Back;
uint8_t Key_One;
uint8_t Key_Two;
uint8_t Key_Three;
uint8_t Key_Four;
uint8_t Key_Trig;
uint8_t Key_Puls;
uint8_t Key_HV;
uint8_t IC2_New;
uint8_t IC2_Old;
uint8_t IC3_New;
uint8_t IC3_Old;
uint8_t Key_Action;
volatile uint16_t intcount = 10; // used to count down 1 sec or 1 msec intervals
uint8_t cursor_x;
uint8_t cursor_y;










// The list of Define-Statements goes here
//#define F_CPU 16000000L
#define reload_time 1000

// Defines Port A, Bits 0 and 2 are Inputs

// Port A, Bit 1 auf High setzen
#define Ena_HV_Relay_ON			PORTA |= (1 << PA1)
// Port A, Bit 1 auf Low setzen
#define Ena_HV_Relay_OFF		PORTA &= ~(1 << PA1)

// Port A, Bit 3 auf High setzen
#define Led_48_ON			PORTA |= (1 << PA3)
// Port A, Bit 3 auf Low setzen
#define Led_48_OFF			PORTA &= ~(1 << PA3)

// Port A, Bit 4 auf High setzen
#define Led_49_ON			PORTA |= (1 << PA4)
// Port A, Bit 4 auf Low setzen
#define Led_49_OFF			PORTA &= ~(1 << PA4)

// Port A, Bit 5 auf High setzen
#define Led_50_ON			PORTA |= (1 << PA5)
// Port A, Bit 5 auf Low setzen
#define Led_50_OFF			PORTA &= ~(1 << PA5)

// Port A, Bit 6 auf High setzen
#define Led_51_ON			PORTA |= (1 << PA6)
// Port A, Bit 6 auf Low setzen
#define Led_51_OFF			PORTA &= ~(1 << PA6)

// Port A, Bit 7 auf High setzen
#define Led_52_ON			PORTA |= (1 << PA7)
// Port A, Bit 7 auf Low setzen
#define Led_52_OFF			PORTA &= ~(1 << PA7)

// Defines Port B, Bits 1, 5 and 7 are Inputs, Bit 3 is MISO only

// Port B, Bit 0 auf High setzen
#define Trig_Ena_ON			PORTB |= (1 << PB0)
// Port B, Bit 0 auf Low setzen
#define Trig_Ena_OFF		PORTB &= ~(1 << PB0)

// Port B, Bit 2 auf High setzen
#define Sample_End_ON		PORTB |= (1 << PB2)
// Port B, Bit 2 auf Low setzen
#define Sample_End_OFF		PORTB &= ~(1 << PB2)

// Port B, Bit 4 auf High setzen
#define HV_Cur_ADC__Conv_ON		PORTB |= (1 << PB4)
// Port B, Bit 4 auf Low setzen
#define HV_Cur_ADC__Conv_OFF	PORTB &= ~(1 << PB4)

// Port B, Bit 6 auf High setzen
#define Puls_ADC__Conv_ON		PORTB |= (1 << PB6)
// Port B, Bit 6 auf Low setzen
#define Puls_ADC__Conv_OFF		PORTB &= ~(1 << PB6)

// Defines Port C, all outputs

// Port C, Bit 0 auf High setzen
#define DB0_ON		PORTC |= (1 << PC0)
// Port C, Bit 0 auf Low setzen
#define DB0_OFF		PORTC &= ~(1 << PC0)

// Port C, Bit 1 auf High setzen
#define DB1_ON		PORTC |= (1 << PC1)
// Port C, Bit 1 auf Low setzen
#define DB1_OFF		PORTC &= ~(1 << PC1)

// Port C, Bit 2 auf High setzen
#define DB2_ON		PORTC |= (1 << PC2)
// Port C, Bit 2 auf Low setzen
#define DB2_OFF		PORTC &= ~(1 << PC2)

// Port C, Bit 3 auf High setzen
#define DB3_ON		PORTC |= (1 << PC3)
// Port C, Bit 3 auf Low setzen
#define DB3_OFF		PORTC &= ~(1 << PC3)

// Port C, Bit 4 auf High setzen
#define DB4_ON		PORTC |= (1 << PC4)
// Port C, Bit 4 auf Low setzen
#define DB4_OFF		PORTC &= ~(1 << PC4)

// Port C, Bit 5 auf High setzen
#define DB5_ON		PORTC |= (1 << PC5)
// Port C, Bit 5 auf Low setzen
#define DB5_OFF		PORTC &= ~(1 << PC5)

// Port C, Bit 6 auf High setzen
#define DB6_ON		PORTC |= (1 << PC6)
// Port C, Bit 6 auf Low setzen
#define DB6_OFF		PORTC &= ~(1 << PC6)

// Port C, Bit 7 auf High setzen
#define DB7_ON		PORTC |= (1 << PC7)
// Port C, Bit 7 auf Low setzen
#define DB7_OFF		PORTC &= ~(1 << PC7)

// Defines Port D, Bit 5 is an input

// Port D, Bit 0 auf High setzen
#define CLK595_ON		PORTD |= (1 << PD0)
// Port D, Bit 0 auf Low setzen
#define CLK595_OFF		PORTD &= ~(1 << PD0)

// Port D, Bit 1 auf High setzen
#define DATA595_ON		PORTD |= (1 << PD1)
// Port D, Bit 1 auf Low setzen
#define DATA595_OFF		PORTD &= ~(1 << PD1)

// Port D, Bit 2 auf High setzen
#define STR595_ON		PORTD |= (1 << PD2)
// Port D, Bit 2 auf Low setzen
#define STR595_OFF		PORTD &= ~(1 << PD2)

// Port D, Bit 3 auf High setzen
#define STR165_ON		PORTD |= (1 << PD3)
// Port D, Bit 3 auf Low setzen
#define STR165_OFF		PORTD &= ~(1 << PD3)

// Port D, Bit 4 auf High setzen
#define SCK165_ON		PORTD |= (1 << PD4)
// Port D, Bit 4 auf Low setzen
#define SCK165_OFF		PORTD &= ~(1 << PD4)

// Port D, Bit 6 auf High setzen
#define E_ON		PORTD |= (1 << PD6)
// Port D, Bit 6 auf Low setzen
#define E_OFF		PORTD &= ~(1 << PD6)

// Port D, Bit 7 auf High setzen
#define RS_ON		PORTD |= (1 << PD7)
// Port D, Bit 7 auf Low setzen
#define RS_OFF		PORTD &= ~(1 << PD7)

// Defines Port E, Bits 0 and 1 are serial connection, Bit 7 is an input

// Port E, Bit 2 auf High setzen
#define DAC_DIN_ON		PORTE |= (1 << PE2)
// Port E, Bit 2 auf Low setzen
#define DAC_DIN_OFF		PORTE &= ~(1 << PE2)

// Port E, Bit 3 auf High setzen
#define DAC_SCLK_ON		PORTE |= (1 << PE3)
// Port E, Bit 3 auf Low setzen
#define DAC_SCLK_OFF	PORTE &= ~(1 << PE3)

// Port E, Bit 4 auf High setzen
#define Time_DAC__Sync_ON		PORTE |= (1 << PE4)
// Port E, Bit 4 auf Low setzen
#define Time_DAC__Sync_OFF		PORTE &= ~(1 << PE4)

// Port E, Bit 5 auf High setzen
#define HV_DAC__Sync_ON			PORTE |= (1 << PE5)
// Port E, Bit 5 auf Low setzen
#define HV_DAC__Sync_OFF		PORTE &= ~(1 << PE5)

// Port E, Bit 6 auf High setzen
#define Puls_Cur_ADC__Conv_ON		PORTE |= (1 << PE6)
// Port E, Bit 6 auf Low setzen
#define Puls_Cur_ADC__Conv_OFF		PORTE &= ~(1 << PE6)

// Defines Port F, Bits 1 and 4 are inputs

// Port F, Bit 0 auf High setzen
#define ADC_CLK_ON		PORTF |= (1 << PF0)
// Port F, Bit 0 auf Low setzen
#define ADC_CLK_OFF		PORTF &= ~(1 << PF0)

// Port F, Bit 2 auf High setzen
#define Samp_ADC__Conv_ON		PORTF |= (1 << PF2)
// Port F, Bit 2 auf Low setzen
#define Samp_ADC__Conv_OFF		PORTF &= ~(1 << PF2)

// Port F, Bit 3 auf High setzen
#define HV_ADC__Conv_ON		PORTF |= (1 << PF3)
// Port F, Bit 3 auf Low setzen
#define HV_ADC__Conv_OFF	PORTF &= ~(1 << PF3)

// Port F, Bit 5 auf High setzen
#define Count_Clock_ON		PORTF |= (1 << PF5)
// Port F, Bit 5 auf Low setzen
#define Count_Clock_OFF		PORTF &= ~(1 << PF5)

// Port F, Bit 6 auf High setzen
#define Count_Serdata_ON		PORTF |= (1 << PF6)
// Port F, Bit 6 auf Low setzen
#define Count_Serdata_OFF		PORTF &= ~(1 << PF6)

// Port F, Bit 7 auf High setzen
#define Count_Strobe_ON		PORTF |= (1 << PF7)
// Port F, Bit 7 auf Low setzen
#define Count_Strobe_OFF	PORTF &= ~(1 << PF7)

// Defines Port G, all 5 Bits are outputs

// Port G, Bit 0 auf High setzen
#define HV_On__Off_ON		PORTG |= (1 << PG0)
// Port G, Bit 0 auf Low setzen
#define HV_On__Off_OFF		PORTG &= ~(1 << PG0)

// Port G, Bit 1 auf High setzen
#define HV_Pol_SW_ON		PORTG |= (1 << PG1)
// Port G, Bit 1 auf Low setzen
#define HV_Pol_SW_OFF		PORTG &= ~(1 << PG1)

// Port G, Bit 2 auf High setzen
#define Puls_Inh_ON			PORTG |= (1 << PG2)
// Port G, Bit 2 auf Low setzen
#define Puls_Inh_OFF		PORTG &= ~(1 << PG2)

// Port G, Bit 3 auf High setzen
#define Fault_Off_ON		PORTG |= (1 << PG3)
// Port G, Bit 3 auf Low setzen
#define Fault_Off_OFF		PORTG &= ~(1 << PG3)

// Port G, Bit 4 auf High setzen
#define _Buzzer_ON		PORTG |= (1 << PG4)
// Port G, Bit 4 auf Low setzen
#define _Buzzer_OFF		PORTG &= ~(1 << PG4)

void startup(void)
{
	// To establish the initial states of the outputs to safe values and defined states
	volatile int i;
	//uint8_t ab=0; // for the debugging of WRITEDAC
	int shiftinit[9];
	shiftinit[0]=1; //Just for completeness' sake
	shiftinit[1]=1; //QH
	shiftinit[2]=1; //QG
	shiftinit[3]=1; //QF
	shiftinit[4]=1; //QE
	shiftinit[5]=1; //QD
	shiftinit[6]=1; //QC
	shiftinit[7]=0; //QB
	shiftinit[8]=1; //QA
	
	//beep();
	Ena_HV_Relay_OFF;
	Led_48_OFF;
	Led_49_OFF;
	Led_50_ON;
	Led_51_OFF;
	Led_52_ON;
	Trig_Ena_OFF;
	Sample_End_OFF;
	HV_Cur_ADC__Conv_ON;
	Puls_ADC__Conv_ON;
	HV_ADC__Conv_ON;
	Samp_ADC__Conv_ON;
	ADC_CLK_OFF;
	Count_Clock_OFF;
	Count_Serdata_OFF;
	Count_Strobe_OFF;
	// The display will be initialized later in a separate routine
	STR595_OFF;
	for (i=1;i<=8;i++)
	{
		CLK595_OFF;
		if (shiftinit[i]==1)
		{
			DATA595_ON;
		}
		else
		{
			DATA595_OFF;
		}
		CLK595_ON; // Perhaps some delay needed here
	}
	STR595_ON; // perhaps some delay needed here
	STR595_OFF;
	STR165_OFF;
	STR165_ON;
	SCK165_OFF;
	DAC_DIN_OFF;
	DAC_SCLK_ON;
	Time_DAC__Sync_ON;
	HV_DAC__Sync_ON;
	Puls_Cur_ADC__Conv_ON;
	// Now initialize the DACs for standby operation mode 01
	//Led_48_ON;
	Dac_Puls_Curr_Value = 0;
	
	Dac_Puls_Curr_Mode = 1;
	
	i=wr_voltdac(0);
	Led_49_ON;
	/*
	// voltage DAC U$17 output both channels one at a time
	int j, val, mod;
	tempdac[23] = 0; // starting at ONE, one number higher than in 8552 data sheet!!!
	tempdac[24] = 0;
	tempdac[20] = 0;
	//Led_48_ON;
	if (ab == 0)
	{
		val = Dac_Puls_Curr_Value;
		mod = Dac_Puls_Curr_Mode;
		tempdac[19] = 0;
		tempdac[21] = 1;
		tempdac[22] = 0;
	}
	else
	{
		val = Dac_Sampling_Value;
		mod = Dac_Sampling_Mode;
		tempdac[19] = 1;
		tempdac[21] = 0;
		tempdac[22] = 1;
	}
	//Led_48_ON;
	for (j = 1; j <= 16;j++)
	{
		if (val % 2 == 0) // Put the 16 bits into single vars
		{
			tempdac[j] = 0;
		}
		else
		{
			tempdac[j] = 1;
		}
		val = int(val/2);
	}
	//Led_48_ON;
	for (j = 17;j <= 18;j++) // get the 2 mod bits sorted
	{
		if (mod % 2 ==0)
		{
			tempdac[j] = 0;
		}
		else
		{
			tempdac[j] = 1;
		}
	} // The bits are all in place here
	//Led_48_ON;
	HV_DAC__Sync_ON;
	_delay_us(3);
	DAC_SCLK_ON;
	_delay_us(3);
	HV_DAC__Sync_OFF;
	//Led_48_ON;
	for (j = 24;j >=1;j--)
	{
		if (tempdac[j] == 1) // possibly some waiting here...
		{
			DAC_DIN_ON;
		}
		else
		{
			DAC_DIN_OFF;
		}
		DAC_SCLK_OFF;
		_delay_us(3);
		DAC_SCLK_ON; // most probable some waiting here
		_delay_us(3);
	}
	//Led_48_ON;
	HV_DAC__Sync_ON; // Value taken over and converted
	Led_48_ON;
	*/
	Dac_Sampling_Value = 0;
	Dac_Sampling_Mode = 1;
	i=wr_voltdac(1);
	//Led_49_ON;
	Dac_HV_Value = 0;
	Dac_HV_Mode = 1;
	wr_sampdac(0);
	Dac_Puls_Volt_Value = 0;
	Dac_Puls_Volt_Mode = 1;
	wr_sampdac(1);
	//Led_48_ON;
	disp_init();
	disp_erase();
	strcpy(wtext,"Hello World");
	disp_write(wtext,1,1);
	strcpy(wtext,"First");
	disp_write(wtext,2,5);
	Set_Trig_Imp = 0;
	HV_Sup_Rel = 0; // will be connected to Ena_HV_Relay, was a mistake in the schematic!
	HV_Meas_Pol = 0; // non-inverted
	C_Big = 0;
	C_Mid = 0;
	Behl__Inh = 0;
	count_val = 32000; // just as an example
	//Led_48_ON;
	sr_write();
	HV_Pol_SW_OFF;
	Fault_Off_OFF;
	//Led_48_ON;
	_Buzzer_OFF;
	beep();
	_Buzzer_OFF;
	//Led_48_ON;
	HV_On__Off_OFF;
	Puls_Inh_OFF;
	wait_msec(5); // macht den Code um ca 100 Byte kleiner --> warum???
	_delay_ms(10);
	win_develop();
	
	
	return;
}

int wr_voltdac(uint8_t ab)
{
	// voltage DAC U$17 output both channels one at a time
	volatile int j, val, modus;
	tempdac[23] = 0; // starting at ONE, one number higher than in 8552 data sheet!!!
	tempdac[24] = 0;
	tempdac[20] = 0;
	//Led_48_ON;
	if (ab == 0)
	{
		val = Dac_Puls_Curr_Value;
		modus = Dac_Puls_Curr_Mode;
		tempdac[19] = 0;
		tempdac[21] = 1;
		tempdac[22] = 0;
	}
	else
	{
		val = Dac_Sampling_Value;
		modus = Dac_Sampling_Mode;
		tempdac[19] = 1;
		tempdac[21] = 0;
		tempdac[22] = 1;
	}
	//Led_48_ON;
	for (j = 1; j <= 16;j++)
	{
		if (val % 2 == 0) // Put the 16 bits into single vars
		{
			tempdac[j] = 0;
		}
		else
		{
			tempdac[j] = 1;
		}
		val = int(val/2);
	}
	//Led_48_ON;
	for (j = 17;j <= 18;j++) // get the 2 mod bits sorted
	{
		if (modus % 2 ==0)
		{
			tempdac[j] = 0;
		}
		else
		{
			tempdac[j] = 1;
		}
	} // The bits are all in place here
	//Led_48_ON;
	HV_DAC__Sync_ON;
	_delay_us(3);
	DAC_SCLK_ON;
	_delay_us(3);
	HV_DAC__Sync_OFF;
	//Led_48_ON;
	
	/*for (j = 1;j <=24;j++)
	{
		if (tempdac[25-j] == 1) // possibly some waiting here...
		{
			DAC_DIN_ON;
		}
		else
		{
			DAC_DIN_OFF;
		}	
		DAC_SCLK_OFF;
		_delay_us(3);
		DAC_SCLK_ON; // most probable some waiting here
		_delay_us(3);
	} */
	if (tempdac[24] == 1) // possibly some waiting here...
	{
		DAC_DIN_ON;
	}
	else
	{
		DAC_DIN_OFF;
	}
	DAC_SCLK_OFF;
	_delay_us(3);
	DAC_SCLK_ON; // most probable some waiting here
	_delay_us(3);
	if (tempdac[23] == 1) // possibly some waiting here...
	{
		DAC_DIN_ON;
	}
	else
	{
		DAC_DIN_OFF;
	}
	DAC_SCLK_OFF;
	_delay_us(3);
	DAC_SCLK_ON; // most probable some waiting here
	_delay_us(3);
	if (tempdac[22] == 1) // possibly some waiting here...
	{
		DAC_DIN_ON;
	}
	else
	{
		DAC_DIN_OFF;
	}
	DAC_SCLK_OFF;
	_delay_us(3);
	DAC_SCLK_ON; // most probable some waiting here
	_delay_us(3);
	if (tempdac[21] == 1) // possibly some waiting here...
	{
		DAC_DIN_ON;
	}
	else
	{
		DAC_DIN_OFF;
	}
	DAC_SCLK_OFF;
	_delay_us(3);
	DAC_SCLK_ON; // most probable some waiting here
	_delay_us(3);
	if (tempdac[20] == 1) // possibly some waiting here...
	{
		DAC_DIN_ON;
	}
	else
	{
		DAC_DIN_OFF;
	}
	DAC_SCLK_OFF;
	_delay_us(3);
	DAC_SCLK_ON; // most probable some waiting here
	_delay_us(3);
	if (tempdac[19] == 1) // possibly some waiting here...
	{
		DAC_DIN_ON;
	}
	else
	{
		DAC_DIN_OFF;
	}
	DAC_SCLK_OFF;
	_delay_us(3);
	DAC_SCLK_ON; // most probable some waiting here
	_delay_us(3);
	if (tempdac[18] == 1) // possibly some waiting here...
	{
		DAC_DIN_ON;
	}
	else
	{
		DAC_DIN_OFF;
	}
	DAC_SCLK_OFF;
	_delay_us(3);
	DAC_SCLK_ON; // most probable some waiting here
	_delay_us(3);
	if (tempdac[17] == 1) // possibly some waiting here...
	{
		DAC_DIN_ON;
	}
	else
	{
		DAC_DIN_OFF;
	}
	DAC_SCLK_OFF;
	_delay_us(3);
	DAC_SCLK_ON; // most probable some waiting here
	_delay_us(3);
	if (tempdac[16] == 1) // possibly some waiting here...
	{
		DAC_DIN_ON;
	}
	else
	{
		DAC_DIN_OFF;
	}
	DAC_SCLK_OFF;
	_delay_us(3);
	DAC_SCLK_ON; // most probable some waiting here
	_delay_us(3);
	if (tempdac[15] == 1) // possibly some waiting here...
	{
		DAC_DIN_ON;
	}
	else
	{
		DAC_DIN_OFF;
	}
	DAC_SCLK_OFF;
	_delay_us(3);
	DAC_SCLK_ON; // most probable some waiting here
	_delay_us(3);
	if (tempdac[14] == 1) // possibly some waiting here...
	{
		DAC_DIN_ON;
	}
	else
	{
		DAC_DIN_OFF;
	}
	DAC_SCLK_OFF;
	_delay_us(3);
	DAC_SCLK_ON; // most probable some waiting here
	_delay_us(3);
	if (tempdac[13] == 1) // possibly some waiting here...
	{
		DAC_DIN_ON;
	}
	else
	{
		DAC_DIN_OFF;
	}
	DAC_SCLK_OFF;
	_delay_us(3);
	DAC_SCLK_ON; // most probable some waiting here
	_delay_us(3);
	if (tempdac[12] == 1) // possibly some waiting here...
	{
		DAC_DIN_ON;
	}
	else
	{
		DAC_DIN_OFF;
	}
	DAC_SCLK_OFF;
	_delay_us(3);
	DAC_SCLK_ON; // most probable some waiting here
	_delay_us(3);
	if (tempdac[11] == 1) // possibly some waiting here...
	{
		DAC_DIN_ON;
	}
	else
	{
		DAC_DIN_OFF;
	}
	DAC_SCLK_OFF;
	_delay_us(3);
	DAC_SCLK_ON; // most probable some waiting here
	_delay_us(3);
	if (tempdac[10] == 1) // possibly some waiting here...
	{
		DAC_DIN_ON;
	}
	else
	{
		DAC_DIN_OFF;
	}
	DAC_SCLK_OFF;
	_delay_us(3);
	DAC_SCLK_ON; // most probable some waiting here
	_delay_us(3);
	if (tempdac[9] == 1) // possibly some waiting here...
	{
		DAC_DIN_ON;
	}
	else
	{
		DAC_DIN_OFF;
	}
	DAC_SCLK_OFF;
	_delay_us(3);
	DAC_SCLK_ON; // most probable some waiting here
	_delay_us(3);
	if (tempdac[8] == 1) // possibly some waiting here...
	{
		DAC_DIN_ON;
	}
	else
	{
		DAC_DIN_OFF;
	}
	DAC_SCLK_OFF;
	_delay_us(3);
	DAC_SCLK_ON; // most probable some waiting here
	_delay_us(3);
	if (tempdac[7] == 1) // possibly some waiting here...
	{
		DAC_DIN_ON;
	}
	else
	{
		DAC_DIN_OFF;
	}
	DAC_SCLK_OFF;
	_delay_us(3);
	DAC_SCLK_ON; // most probable some waiting here
	_delay_us(3);
	if (tempdac[6] == 1) // possibly some waiting here...
	{
		DAC_DIN_ON;
	}
	else
	{
		DAC_DIN_OFF;
	}
	DAC_SCLK_OFF;
	_delay_us(3);
	DAC_SCLK_ON; // most probable some waiting here
	_delay_us(3);
	if (tempdac[5] == 1) // possibly some waiting here...
	{
		DAC_DIN_ON;
	}
	else
	{
		DAC_DIN_OFF;
	}
	DAC_SCLK_OFF;
	_delay_us(3);
	DAC_SCLK_ON; // most probable some waiting here
	_delay_us(3);
	if (tempdac[4] == 1) // possibly some waiting here...
	{
		DAC_DIN_ON;
	}
	else
	{
		DAC_DIN_OFF;
	}
	DAC_SCLK_OFF;
	_delay_us(3);
	DAC_SCLK_ON; // most probable some waiting here
	_delay_us(3);
	if (tempdac[3] == 1) // possibly some waiting here...
	{
		DAC_DIN_ON;
	}
	else
	{
		DAC_DIN_OFF;
	}
	DAC_SCLK_OFF;
	_delay_us(3);
	DAC_SCLK_ON; // most probable some waiting here
	_delay_us(3);
	if (tempdac[2] == 1) // possibly some waiting here...
	{
		DAC_DIN_ON;
	}
	else
	{
		DAC_DIN_OFF;
	}
	DAC_SCLK_OFF;
	_delay_us(3);
	DAC_SCLK_ON; // most probable some waiting here
	_delay_us(3);
	if (tempdac[1] == 1) // possibly some waiting here...
	{
		DAC_DIN_ON;
	}
	else
	{
		DAC_DIN_OFF;
	}
	DAC_SCLK_OFF;
	_delay_us(10);
	DAC_SCLK_ON; // most probable some waiting here
	_delay_us(10);
	
	
	//Led_48_ON;
	//vr_volt_bb();
	//j = 24;
	//Led_48_ON;
	/*while (j >= 1)
	{
		if (tempdac[j] == 1) // possibly some waiting here...
		{
			DAC_DIN_ON;
		}
		else
		{
			DAC_DIN_OFF;
		}
		j --;
		_delay_us(3);
		DAC_SCLK_OFF;
		_delay_us(3);
		DAC_SCLK_ON; // most probable some waiting here
		_delay_us(3);
	}*/
	//Led_48_ON;
	HV_DAC__Sync_ON; // Value taken over and converted
	_delay_us(10);
	
	Led_51_ON;
	return 1;
}

void wr_sampdac(uint8_t ab)
{
	// sampling DAC U$21 output both channels one at a time
	int i, val, mod;
	tempdac[23] = 0; // starting at ONE, one number highwer than in 8552 data sheet!!!
	tempdac[24] = 0;
	tempdac[20] = 0;
	
	if (ab == 0)
	{
		val = Dac_HV_Value;
		mod = Dac_HV_Mode;
		tempdac[19] = 0;
		tempdac[21] = 1;
		tempdac[22] = 0;
	}
	else
	{
		val = Dac_Puls_Volt_Value;
		mod = Dac_Puls_Volt_Mode;
		tempdac[19] = 1;
		tempdac[21] = 0;
		tempdac[22] = 1;
	}
	
	for (i = 1; i <= 16;i++)
	{
		if (val % 2 == 0) // Put the 16 bits into single vars
		{
			tempdac[i] = 0;
		}
		else
		{
			tempdac[i] = 1;
		}
		val = int(val/2);
	}
	for (i = 17;i <= 18;i++) // get the 2 mod bits sorted
	{
		if (mod % 2 ==0)
		{
			tempdac[i] = 0;
		}
		else
		{
			tempdac[i] = 1;
		}
	} // The bits are all in place here
	Time_DAC__Sync_ON;
	DAC_SCLK_ON;
	Time_DAC__Sync_OFF;
	for (i = 24;i >=1;i--)
	{
		if (tempdac[i] == 1) // possibly some waiting here...
		{
			DAC_DIN_ON;
		}
		else
		{
			DAC_DIN_OFF;
		}
		DAC_SCLK_OFF;
		DAC_SCLK_ON; // most probable some waiting here
	}
	Time_DAC__Sync_ON; // Value taken over and converted
	return;
}

void disp_erase(void)
{
	// will erase the display content and prepare for next display
	RS_OFF;
	E_ON;
	PORTC = 0b00000001; // erase display, cursor at row 1, col 1
	//shortwait(1);
	_delay_us(18);
	E_OFF;
	//shortwait(1);
	_delay_us(18);
	return;
}
void disp_init(void)
{
	// will initialize and set up the display for operation
	RS_OFF;
	// set function of display
	E_ON;
	PORTC = 0b00111000; // 8 bit mode, two-row display
	//shortwait(1);
	_delay_us(18);
	E_OFF;
	//shortwait(1);
	_delay_us(18);
	// Display ON/OFF
	E_ON;
	PORTC = 0b00001100; // display on, no cursor, cursor not blinking
	E_OFF;
	//shortwait(1);
	_delay_us(18);
	// Clear display
	E_ON;
	PORTC = 0b00000001;
	E_OFF;
	//shortwait(1);
	_delay_us(18);
	// Set entry mode
	E_ON;
	PORTC = 0b00000110; //Cursor auto-increment
	E_OFF;
	return;
}

void disp_write(char wtext[26], int row, int column)
{
	// write a number of chars to the display
	uint8_t i;
	
	RS_OFF;
	E_ON;
	// set DRAM address
	switch (row)
	{
	case 1: PORTC = 0x80 + (column - 1);
		break;
	case 2: PORTC = 0xc0 + (column - 1);
		break;
	case 3: PORTC = 0x94 + (column - 1);
		break;
	case 4: PORTC = 0xd4 + (column - 1);
		break;	
	}
	//shortwait(1);
	_delay_us(18);
	E_OFF;
	RS_ON;
	for (i = 0; i < strlen(wtext); i++)
	{
		E_ON;
		//shortwait(1);
		_delay_us(18);
		switch (wtext[i])
		{
		case '':
		case '': PORTC = 0b11110101;
			break;
		case '':
		case '': PORTC = 0b11100001;
			break;
		case '':
		case '': PORTC = 0b11101111;
			break;
		case '': PORTC = 0b11100100;
			break;
		case '': PORTC = 0b11100010;
			break;
		case '@': PORTC = 0b11110110;
			break;
		case '\0': return;
		default: PORTC = wtext[i];
			break;		
		}
		E_OFF;
		if (i > 25) 
		{
			break;
		}
	}
	RS_OFF;
	return;
}

void shortwait(unsigned int delay)
{
	// short wait 18 sec, needed for display
	sh_wait = delay;
	while (sh_wait);
	return;
}

ISR (TIMER0_OVF_vect)
{
	intcount--; // 1 msec
	return;
}

ISR (TIMER2_OVF_vect)
{
	sh_wait--;
	return;
}

ISR (TIMER1_OVF_vect)
{
	timecount--;
	delaycount--;
	int i, t, n, o;
	
	Led_48_ON;
	// Summing for time scale h:m:s
	if (timecount == 0)
	{
		sec++;
		if (sec == 60)
		{
			sec = 0;
			min++;
			if (min == 60)
			{
				min = 0;
				hour++;
				if (hour == 24)
				{
					hour = 0;
				}
			}
		}
		timecount = reload_time;
	}
	// Looking for a change of signals from keyboard
	SCK165_OFF;
	STR165_OFF;
	STR165_ON;
	STR165_OFF;
	IC2_New = 0;
	IC3_New = 0;
	t = 128;
	for (i = 1;i <= 8;i++)
	{
		SCK165_ON;
		if ((PIND & (1 << PIND5)) == 1) // read 165dat
		{
			IC3_New += t;
		} 
		t /= 2;
		SCK165_OFF;
	}
	t = 128;
	for (i = 1; i <= 8;i++)
	{
		SCK165_ON;
		if ((PIND & (1 << PIND5)) == 1)
		{
			IC2_New += t;
		}
		t /= 2;
		SCK165_OFF;
	}
	if ((IC2_Old != IC2_New) || (IC3_Old != IC3_New))
	{
		// some bit has changed from the keyboard
		Key_Action = 1;
		n = IC2_New;
		o = IC2_Old;
		// Find out which bits have been changed
		if (n % 2 != o % 2)
		{
			Key_Four = 1;
		}
		n = int (n/2);
		o = int (o/2);
		if (n % 2 != o % 2)
		{
			Key_Three = 1;
		}
		n = int (n/2);
		o = int (o/2);
		if (n % 2 != o % 2)
		{
			Key_Two = 1;
		}
		n = int (n/2);
		o = int (o/2);
		if (n % 2 != o % 2)
		{
			Key_One = 1;
		}
		n = int (n/2);
		o = int (o/2);
		if (n % 2 != o % 2)
		{
			Key_Back = 1;
		}
		n = int (n/2);
		o = int (o/2);
		if (n % 2 != o % 2)
		{
			Key_Trig = 1;
		}
		n = int (n/2);
		o = int (o/2);
		if (n % 2 != o % 2)
		{
			Key_Puls = 1;
		}
		n = int (n/2);
		o = int (o/2);
		if (n % 2 != o % 2)
		{
			Key_HV = 1;
		}
		n = IC3_New;
		o = IC3_Old;
		n = int (n/16);
		o = int (o/16);
		n = int (n/2);
		o = int (o/2);
		if (n % 2 != o % 2)
		{
			But_Local = 1;
		}
		n = int (n/2);
		o = int (o/2);
		if (n % 2 != o % 2)
		{
			Enc_But = 1;
		}
		n = int (n/2);
		o = int (o/2);
		if (n % 2 != o % 2)
		{
			Enc_B = 1;
		}
		n = int (n/2);
		o = int (o/2);
		if (n % 2 != o % 2)
		{
			Enc_A = 1;
		}
		
		
	}
	
	
	
	Led_48_OFF;
	return;
}

void sr_write(void) 
{
	// this routine will get and sort the shift register values for the counter
	// as well as outputting the values of the third shift register
	int i;
	if ((count_val % 2) == 0)
	{
		Sreg_Sort[4] = 0;
	}
	else
	{
		Sreg_Sort[4] = 1;
	}
	count_val = int(count_val / 2);
	if ((count_val % 2) == 0)
	{
		Sreg_Sort[3] = 0;
	}
	else
	{
		Sreg_Sort[3] = 1;
	}
	count_val = int(count_val / 2);
	if ((count_val % 2) == 0)
	{
		Sreg_Sort[2] = 0;
	}
	else
	{
		Sreg_Sort[2] = 1;
	}
	count_val = int(count_val / 2);
	if ((count_val % 2) == 0)
	{
		Sreg_Sort[1] = 0;
	}
	else
	{
		Sreg_Sort[1] = 1;
	}
	count_val = int(count_val / 2);
	if ((count_val % 2) == 0)
	{
		Sreg_Sort[8] = 0;
	}
	else
	{
		Sreg_Sort[8] = 1;
	}
	count_val = int(count_val / 2);
	if ((count_val % 2) == 0)
	{
		Sreg_Sort[7] = 0;
	}
	else
	{
		Sreg_Sort[7] = 1;
	}
	count_val = int(count_val / 2);
	if ((count_val % 2) == 0)
	{
		Sreg_Sort[6] = 0;
	}
	else
	{
		Sreg_Sort[6] = 1;
	}
	count_val = int(count_val / 2);
	if ((count_val % 2) == 0)
	{
		Sreg_Sort[5] = 0;
	}
	else
	{
		Sreg_Sort[5] = 1;
	}
	count_val = int(count_val / 2);
	if ((count_val % 2) == 0)
	{
		Sreg_Sort[12] = 0;
	}
	else
	{
		Sreg_Sort[12] = 1;
	}
	count_val = int(count_val / 2);
	if ((count_val % 2) == 0)
	{
		Sreg_Sort[11] = 0;
	}
	else
	{
		Sreg_Sort[11] = 1;
	}
	count_val = int(count_val / 2);
	if ((count_val % 2) == 0)
	{
		Sreg_Sort[10] = 0;
	}
	else
	{
		Sreg_Sort[10] = 1;
	}
	count_val = int(count_val / 2);
	if ((count_val % 2) == 0)
	{
		Sreg_Sort[9] = 0;
	}
	else
	{
		Sreg_Sort[9] = 1;
	}
	count_val = int(count_val / 2);
	if ((count_val % 2) == 0)
	{
		Sreg_Sort[16] = 0;
	}
	else
	{
		Sreg_Sort[16] = 1;
	}
	count_val = int(count_val / 2);
	if ((count_val % 2) == 0)
	{
		Sreg_Sort[15] = 0;
	}
	else
	{
		Sreg_Sort[15] = 1;
	}
	count_val = int(count_val / 2);
	if ((count_val % 2) == 0)
	{
		Sreg_Sort[14] = 0;
	}
	else
	{
		Sreg_Sort[14] = 1;
	}
	count_val = int(count_val / 2);
	if ((count_val % 2) == 0)
	{
		Sreg_Sort[13] = 0;
	}
	else
	{
		Sreg_Sort[13] = 1;
	}
	count_val = int(count_val / 2);
	// two dummy bytes first
	Count_Clock_OFF;
	Count_Clock_ON; // first bit
	Count_Clock_OFF;
	Count_Clock_ON; // second bit
	Count_Clock_OFF;
	if (Behl__Inh == 1)
	{
		Count_Serdata_ON;
	} 
	else
	{
		Count_Serdata_OFF;
	}
	Count_Clock_ON;
	Count_Clock_OFF;
	if (C_Mid == 1)
	{
		Count_Serdata_ON;
	} 
	else
	{
		Count_Serdata_OFF;
	}
	Count_Clock_ON;
	Count_Clock_OFF;
	if (C_Big == 1)
	{
		Count_Serdata_ON;
	} 
	else
	{
		Count_Serdata_OFF;
	}
	Count_Clock_ON;
	Count_Clock_OFF;
	if (HV_Meas_Pol == 1)
	{
		Count_Serdata_ON;
	} 
	else
	{
		Count_Serdata_OFF;
	}
	Count_Clock_ON;
	Count_Clock_OFF;
	if (HV_Sup_Rel == 1)
	{
		Count_Serdata_ON;
	} 
	else
	{
		Count_Serdata_OFF;
	}
	Count_Clock_ON;
	Count_Clock_OFF;
	if (Set_Trig_Imp == 1)
	{
		Count_Serdata_ON;
	} 
	else
	{
		Count_Serdata_OFF;
	}
	Count_Clock_ON;
	Count_Clock_OFF;
	for (i=16; i>=1; i--)
	{
		if (Sreg_Sort[i] == 1)
		{
			Count_Serdata_ON;
		} 
		else
		{
			Count_Serdata_OFF;
		}
		Count_Clock_ON;
		Count_Clock_OFF;
	}
	Count_Strobe_ON;
	shortwait(1);
	Count_Strobe_OFF;
	return;
}

void wait_msec(uint16_t mscount)
{
	intcount = mscount;
	while (intcount != 0)
	{
		mscount++;
		mscount = mscount -1;
	}
	return;
}

void wait_sec(uint16_t secount)
{
	while (secount != 0)
	{
		intcount = 1000; // to set one second
		while (intcount != 0)
		{
			secount ++;
			secount = secount -1;
		}
		secount--;
	}
	return;
}

void win_develop(void)
{
	// function to evaluate the vars needed for some basic adjustments and screen displays
	// First establish the static part of the display
	E_ON;
	PORTC = 0b00001111; // Cursor is switched on and blinking
	E_OFF;
	disp_erase();
	_delay_us(18);
	strcpy(wtext,"Adj HV");
	disp_write(wtext,1,1);
	strcpy(wtext,"  PULS");
	disp_write(wtext,2,1);
	strcpy(wtext,"  TIME");
	disp_write(wtext,3,1);
	strcpy(wtext," up  down left right");
	disp_write(wtext,4,1);
	// Do the variables exist for set and actual values of the voltages??
	Dac_Puls_Volt_Value = 0;
	Dac_HV_Value = 0;
	// The value of count_val has been initialized, not necessary here!
	while (!Key_Back)
	{
		// Now the routine itself for changing and setting the values
		
		
		
	}
	
}

void beep()
{
	uint8_t i;
	for (i = 0;i <= 100;i++)
	{
		_Buzzer_ON;
		_delay_ms(1);
		wait_msec(2);
		_Buzzer_OFF;
		_delay_ms(1);
		wait_msec(2);
	}
	return;
}

void vr_volt_bb(void)
{
	int j;
	// versuchen mit "25-j" und aufwrts zhlen
	//j = 24;
	//while (j >= 1)
	for (j = 1; j <= 24; j++)
	{
		if (tempdac[25-j] >= 1) // possibly some waiting here...
		{
			//DAC_DIN_ON;
			PORTE |= (1 << PE2);
		}
		else
		{
			//DAC_DIN_OFF;
			PORTE &= ~(1 << PE2);
		}
		//j --;
		//_delay_us(3);
		//DAC_SCLK_OFF;
		PORTE &= ~(1 << PE3);
		//_delay_us(3);
		//DAC_SCLK_ON; // most probable some waiting here
		PORTE |= (1 << PE3);
		//_delay_us(3);
	}
	Led_48_ON;
	//Led_51_ON;
	//HV_DAC__Sync_ON; // Value taken over and converted
	PORTE |= (1 << PE5);
	return;
}

int main(void)
{
    while(1)
    {
        //TODO:: Please write your application code 
		
		// set pin functions as in- or output
		DDRA = 0b11111010; // bits 0 and 2 are inputs
		PORTA = 0b00000101; // set pullups on inputs
		
		DDRB = 0b01010101; // bits 1,3,5 and 7 are inputs
		PORTB = 0b10101010; // set pullups on inputs
		
		DDRC = 0b11111111; // all bits of the display are outputs
		PORTC = 0b00000000; // no pullups needed
		
		DDRD = 0b11011111; // only bit 5 is an input
		PORTD = 0b00100000; // set pullups on inputs
		
		DDRE = 0b01111100; // only bit 7 is an input, bits 0 and 1 are the serial port
		PORTE = 0b10000000; // set pullups on inputs
		
		DDRF = 0b11101101; // bits 1 and 4 are inputs
		PORTF = 0b00010010; // set pullups on inputs
		
		DDRG = 0b11111; // all outputs, port not complete!!
		PORTG = 0b00000; // no pullups needed
		
		// set paramaters for serial communication 
		
		
		// prepare the timers
		// timer 0
		TCCR0 = 0x00; // stop timer 0
		TCNT0 = 0x00; // clear timer 0
		TIMSK = 0x01; // overflow timer 0 interrupt enable
		TCCR0 = 0x03; // start timer 0 with 1/32 prescaler // 4sec*256 = ~1 msec
		
		// timer 1
		TCCR1B = 0x00; // stop timer 1
		TCCR1A = 0x00; // no output signals
		
		TCNT1H = 0x00; // clear timer 1
		TCNT1L = 0x00;
		
		OCR1AH = 0x00; // set compare A
		OCR1AL = 0xFA; // ((16 MHz / 64) / 250) = 1 msec timer
		TIMSK = TIMSK | 0x10; // output compare interrupt enable
		TCCR1B = 0x0B; // start timer 1 with clk/64, TOP on OCR1AX
		
		// timer 2
		// clk/8 = 1,085 sec clock rate
		// interrupt every 18 sec with setting 71
		TCCR2 = 0x00;
		TCNT2 = 0x00;
		ASSR = 0;
		
		OCR2 = 71; // set compare A
		TIMSK = TIMSK | 0x80; // compare timer 2 interrupt enable
		TCCR2 = 0x03; // start timer 2 with clk /64
		
		// set serial port 8n1, 19200 Baud
		// UBRR = 51
		uint16_t val_ubrr = 51;
		UBRR0H = (unsigned char) (val_ubrr>>8);
		UBRR0L = (unsigned char) val_ubrr;
		UCSR0B = (1 << RXEN0) | (1 << TXEN0); // enables RX and TX, overrides normal function of pins
		UCSR0C = (0 << USBS0) | (3 << UCSZ00) | (0 << UPM00); // 1 stop bit, 8 data bits, no parity
		
		
		startup();
		
		
    }
}
